/*
 * SPDX-FileCopyrightText: 2025 Nextcloud GmbH and Nextcloud contributors
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#include "foldermantestutils.h"

FolderManTestHelper::FolderManTestHelper(QObject *parent)
    : QObject{parent}
{

}

FolderManTestHelper::~FolderManTestHelper()
{

}
