/*
 * SPDX-FileCopyrightText: 2023 Nextcloud GmbH and Nextcloud contributors
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef FileProviderExt_Bridging_Header_h
#define FileProviderExt_Bridging_Header_h

#import "Services/ClientCommunicationProtocol.h"

#endif /* FileProviderExt_Bridging_Header_h */
